﻿Imports System.Drawing
Imports System.Drawing.Imaging
Imports System.IO

Module Utility

#Region "Analog Clock Functions"
    Public Sub DrawAnalogClock(ByRef g As Graphics, ByVal ClockBmp As Bitmap, ByVal Position As Rectangle, _
        ByVal IA As ImageAttributes, ByVal ClockArrowType As String, ByVal ClockBackArrow As Boolean, _
        ByVal ClockColorHour As Color, ByVal ClockDimHour As Integer, ByVal ClockColorMin As Color, _
        ByVal ClockDimMin As Integer, ByVal ClockColorSec As Color, ByVal ClockDimSec As Integer, ByVal DelayHour As Integer)

        'Calculate the Center of Clock
        Dim CenterClock = New Point(Position.Left + (Position.Width \ 2), Position.Top + (Position.Height \ 2))

        'Create Pens for arrows
        Dim PenHour As New Pen(ClockColorHour, ClockDimHour)
        Dim PenMin As New Pen(ClockColorMin, ClockDimMin)
        Dim PenSec As New Pen(ClockColorSec, ClockDimSec)

        'Calculate Degree of time
        Dim Hour As Integer = CalcTime((Now.AddHours(DelayHour).Hour * 60) + Now.Minute, 720)
        Dim Min As Integer = CalcTime((Now.Minute * 60) + Now.Second, 3600)
        Dim Sec As Integer = CalcTime(Now.Second, 60)

        'Calculate Points of time
        Dim HourPt As Point = PosXY(CenterClock, Position.Width / 4.7, Hour)
        Dim MinPt As Point = PosXY(CenterClock, Position.Width / 3.2, Min)
        Dim SecPt As Point = PosXY(CenterClock, Position.Width / 2.7, Sec)

        'Set transparency of bitmap clock
        IA.SetColorKey(ClockBmp.GetPixel(0, 0), ClockBmp.GetPixel(0, 0))
        'Draw Clock bitmap
        g.DrawImage(ClockBmp, Position, 0, 0, ClockBmp.Width, ClockBmp.Height, GraphicsUnit.Pixel, IA)

        Select Case ClockArrowType.ToLower
            Case "empty arrows"
                g.DrawPolygon(New Pen(PenHour.Color, 1), CreateClockArrow(CenterClock, Position.Width / 4.5, PenHour.Width, Hour, ClockBackArrow))
                g.DrawPolygon(New Pen(PenMin.Color, 1), CreateClockArrow(CenterClock, Position.Width / 3.0, PenMin.Width, Min, ClockBackArrow))
                If PenSec.Width > 0 Then g.DrawPolygon(New Pen(PenSec.Color, 1), CreateClockArrow(CenterClock, Position.Width / 2.5, PenSec.Width, Sec, ClockBackArrow))

            Case "full arrows"
                g.FillPolygon(New SolidBrush(PenHour.Color), CreateClockArrow(CenterClock, Position.Width / 4.5, PenHour.Width, Hour, ClockBackArrow))
                g.FillPolygon(New SolidBrush(PenMin.Color), CreateClockArrow(CenterClock, Position.Width / 3.0, PenMin.Width, Min, ClockBackArrow))
                If PenSec.Width > 0 Then g.FillPolygon(New SolidBrush(PenSec.Color), CreateClockArrow(CenterClock, Position.Width / 2.5, PenSec.Width, Sec, ClockBackArrow))

            Case Else
                If ClockBackArrow Then
                    Dim HourBackPt As Point = PosXY(CenterClock, Position.Width / 12, Hour + 180)
                    Dim MinBackPt As Point = PosXY(CenterClock, Position.Width / 10, Min + 180)
                    Dim SecBackPt As Point = PosXY(CenterClock, Position.Width / 8, Sec + 180)

                    g.DrawLine(PenHour, HourBackPt.X, HourBackPt.Y, HourPt.X, HourPt.Y)
                    g.DrawLine(PenMin, MinBackPt.X, MinBackPt.Y, MinPt.X, MinPt.Y)
                    If PenSec.Width > 0 Then g.DrawLine(PenSec, SecBackPt.X, SecBackPt.Y, SecPt.X, SecPt.Y)
                Else
                    g.DrawLine(PenHour, CenterClock.X, CenterClock.Y, HourPt.X, HourPt.Y)
                    g.DrawLine(PenMin, CenterClock.X, CenterClock.Y, MinPt.X, MinPt.Y)
                    If PenSec.Width > 0 Then g.DrawLine(PenSec, CenterClock.X, CenterClock.Y, SecPt.X, SecPt.Y)
                End If
        End Select

        'Disegno il centro della lancetta dei secondi solo se si devono vedere i secondi
        If PenSec.Width > 0 Then g.FillEllipse(New SolidBrush(PenSec.Color), New Rectangle(CenterClock.X - PenMin.Width, CenterClock.Y - PenMin.Width, PenMin.Width * 2, PenMin.Width * 2))

        'Dispose Pens
        PenHour.Dispose()
        PenHour = Nothing
        PenMin.Dispose()
        PenMin = Nothing
        PenSec.Dispose()
        PenSec = Nothing
    End Sub

    Public Function PosXY(ByVal Center As Point, ByVal Radius As Integer, ByVal Degree As Integer) As Point
        Dim x As Integer = Math.Cos(DegreeToRadian(Degree)) * Radius
        Dim y As Integer = Math.Sin(DegreeToRadian(Degree)) * Radius

        Return New Point(Center.X + x, Center.Y + y)
    End Function

    ' Convert Radian To Degree
    Public Function RadianToDegree(ByVal radian As Double) As Double
        Return (180 / Math.PI) * radian
    End Function

    ' Convert Degree to Radian
    Public Function DegreeToRadian(ByVal degree As Double) As Double
        Return (Math.PI / 180) * degree
    End Function

    Public Function CalcTime(ByVal NumTime As Integer, ByVal Divis As Integer) As Integer
        Dim ret As Integer = NumTime - (Divis \ 4)

        If ret < 0 Then
            ret = Divis - Math.Abs(ret)
        ElseIf ret > Divis Then
            ret = 0 + (ret - Divis)
        End If

        Return (ret * 360) \ Divis
    End Function

    Public Function CreateClockArrow(ByVal Center As Point, ByVal LungArrow As Integer, ByVal LargArrow As Integer, ByVal Degree As Integer, ByVal BackArrow As Boolean) As Point()
        Dim ret() As Point

        If BackArrow Then
            ReDim ret(4)
            ret(0) = PosXY(Center, LungArrow / 5, Degree + 180)
            ret(1) = PosXY(Center, LargArrow, Degree - 90)
            ret(2) = PosXY(Center, LungArrow, Degree)
            ret(3) = PosXY(Center, LargArrow, Degree + 90)
            ret(4) = PosXY(Center, LungArrow / 5, Degree - 180)
        Else
            ReDim ret(2)
            ret(0) = PosXY(Center, LargArrow, Degree - 90)
            ret(1) = PosXY(Center, LungArrow, Degree)
            ret(2) = PosXY(Center, LargArrow, Degree + 90)
        End If

        Return ret
    End Function
#End Region

End Module
