﻿Imports System.Drawing
Imports System.Drawing.Imaging
Imports System.IO
Imports System.Windows.Forms

Public Class Widget
    'Get the directory of this Widget
    Private DirWidget As String = Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().GetName().CodeBase)

    'Scale Device based on VGA Resolution
    Private DimScale As Single = 1

    'Dimension of Widget in VGA size(480x640)
    Private WidthWidget As Integer = 160
    Private HeightWidget As Integer = 160

    'Path of Settings file loaded in LoadWidget
    Private SettingsFile As String

    'Default Variables 
    Private ClockBmp As Bitmap
    Private ClockType As String = "Clock1.png"
    Private ClockArrowType As String = "Normal"
    Private ClockBackArrow As Boolean = True
    Private ClockColorSec As Color = Color.Red
    Private ClockDimSec As Integer = 2
    Private ClockColorMin As Color = Color.Black
    Private ClockDimMin As Integer = 3
    Private ClockColorHour As Color = Color.Black
    Private ClockDimHour As Integer = 4
    Private DelayHour As Integer = 0

#Region "Required Methods in this order"
    'Load variables for this Widget
    Public Sub LoadWidget(ByVal PathSettingsFile As String)
        Try
            'Set the path of Settings File
            SettingsFile = PathSettingsFile

            'Load Settings
            LoadSettings()

            'Load the clock Bitmap
            ClockBmp = New Bitmap(DirWidget & "\Clocks\" & ClockType)

            'Set DimScale
            DimScale = Math.Max(480 / Math.Min(Screen.PrimaryScreen.Bounds.Width, Screen.PrimaryScreen.Bounds.Height), 640 / Math.Max(Screen.PrimaryScreen.Bounds.Width, Screen.PrimaryScreen.Bounds.Height))

        Catch ex As Exception
            MsgBox(ex.Message, MsgBoxStyle.Critical, "Error")
        End Try
    End Sub

    Public Sub DrawWidget(ByRef g As Graphics, ByVal Name As String, ByVal Position As Rectangle, ByVal IA As ImageAttributes, ByVal Inflate As Single)
        Try
            'Draw the Analog Clock
            DrawAnalogClock(g, ClockBmp, Position, IA, ClockArrowType, ClockBackArrow, ClockColorHour, ClockDimHour, ClockColorMin, ClockDimMin, ClockColorSec, ClockDimSec, DelayHour)

        Catch ex As Exception
            MsgBox(ex.Message, MsgBoxStyle.Critical, "Error")
        End Try
    End Sub

    Public Sub DisposeWidget()
        Try
            'Dispose objects
            ClockBmp.Dispose()
            ClockBmp = Nothing

        Catch ex As Exception
            MsgBox(ex.Message, MsgBoxStyle.Critical, "Error")
        End Try
    End Sub

    Public Sub ClickOnWidget(ByVal Position As Rectangle, ByVal PointMouse As Point)
        Try
            'Debug
            'MsgBox(Position.X & "," & Position.Y & "," & Position.Width & "," & Position.Height & "; " & PointMouse.X & "," & PointMouse.Y)

            'If you click on bottom of clock you open setting
            If PointMouse.Y > (Position.Height / 2) Then
                Process.Start("ctlpnl", "cplmain.cpl,16")
            Else
                'If you click in top open the settings
                Using frm As New Settings
                    With frm
                        'Set data to form
                        .LoadData = True

                        'Load files to clock images
                        For Each fil As String In Directory.GetFiles(DirWidget & "\Clocks", "*.png")
                            .ClockType.Items.Add(Path.GetFileName(fil))
                        Next

                        'Set fields form
                        .ClockType.Text = ClockType
                        .ClockArrowType.Text = ClockArrowType
                        .ClockBackArrow.Checked = ClockBackArrow
                        .ClockColorSec.BackColor = ClockColorSec
                        .ClockDimSec.Value = ClockDimSec
                        .ClockColorMin.BackColor = ClockColorMin
                        .ClockDimMin.Value = ClockDimMin
                        .ClockColorHour.BackColor = ClockColorHour
                        .ClockDimHour.Value = ClockDimHour
                        .DelayHour.Value = DelayHour

                        'Data loaded
                        .LoadData = False

                        'Create Clock image
                        .SetClockBmp()

                        'Show form
                        .ShowDialog()

                        'If user click on Ok button, save data
                        If .SaveData Then
                            ClockType = .ClockType.Text
                            ClockArrowType = .ClockArrowType.Text
                            ClockBackArrow = .ClockBackArrow.Checked
                            ClockColorSec = .ClockColorSec.BackColor
                            ClockDimSec = .ClockDimSec.Value
                            ClockColorMin = .ClockColorMin.BackColor
                            ClockDimMin = .ClockDimMin.Value
                            ClockColorHour = .ClockColorHour.BackColor
                            ClockDimHour = .ClockDimHour.Value
                            DelayHour = .DelayHour.Value

                            'Reload bitmap
                            If ClockBmp IsNot Nothing Then
                                ClockBmp.Dispose()
                                ClockBmp = Nothing
                            End If
                            ClockBmp = New Bitmap(DirWidget & "\Clocks\" & ClockType)

                            'Save settings
                            SaveSettings()
                        End If
                    End With
                End Using
            End If
        Catch ex As Exception
            MsgBox(ex.Message, MsgBoxStyle.Critical, "Error")
        End Try
    End Sub

    'Return the dimension of widget
    Public Function GetDimension(ByVal WidthHeight As String) As Integer
        Try
            Select Case WidthHeight.ToLower
                Case "width"
                    Return WidthWidget
                Case "height"
                    Return HeightWidget

            End Select
        Catch ex As Exception
            MsgBox(ex.Message, MsgBoxStyle.Critical, "Error")
        End Try
    End Function

    'Get the vertical scroll on widget
    Public Sub ScrollWidget(ByVal ScrollY As Integer)
        Try

        Catch ex As Exception
            MsgBox(ex.Message, MsgBoxStyle.Critical, "Error")
        End Try
    End Sub
#End Region

#Region "Other Functions"
    'Load the Settings
    Private Sub LoadSettings()
        'If settings file not exist
        If Not File.Exists(SettingsFile) Then
            'I create it saving the default variables
            SaveSettings()
        Else
            'Read the settings file
            Using sr As New StreamReader(SettingsFile)
                'Read first line
                Dim row As String = sr.ReadLine

                'Process all lines
                Do While row IsNot Nothing
                    'If this row isnt empty or it isnt a comment i process it
                    If row.Trim <> String.Empty AndAlso Not row.StartsWith("//") AndAlso Not row.StartsWith("'") Then
                        'Split Setting and Value
                        Dim setting As String = row.ToLower.Split("=")(0)
                        Dim val As String = row.ToLower.Split("=")(1)

                        'Check which value is to set
                        Select Case setting
                            Case "clocktype"
                                ClockType = val

                            Case "clockarrowtype"
                                ClockArrowType = val

                            Case "clockbackarrow"
                                ClockBackArrow = val

                            Case "clockcolorsec"
                                Dim col() As String = val.Split(",")
                                If col.Length = 3 Then ClockColorSec = Color.FromArgb(col(0), col(1), col(2))

                            Case "clockdimsec"
                                ClockDimSec = val

                            Case "clockcolormin"
                                Dim col() As String = val.Split(",")
                                If col.Length = 3 Then ClockColorMin = Color.FromArgb(col(0), col(1), col(2))

                            Case "clockdimmin"
                                ClockDimMin = val

                            Case "clockcolorhour"
                                Dim col() As String = val.Split(",")
                                If col.Length = 3 Then ClockColorHour = Color.FromArgb(col(0), col(1), col(2))

                            Case "clockdimhour"
                                ClockDimHour = val

                            Case "delayhour"
                                DelayHour = val

                        End Select
                    End If

                    'Read next line
                    row = sr.ReadLine
                Loop
            End Using
        End If

    End Sub

    'Save the settings
    Private Sub SaveSettings()
        'Write the settings file
        Using sw As New StreamWriter(SettingsFile, False)
            sw.WriteLine("ClockType=" & ClockType)
            sw.WriteLine("ClockArrowType=" & ClockArrowType)
            sw.WriteLine("ClockBackArrow=" & ClockBackArrow)
            sw.WriteLine("ClockColorSec=" & ClockColorSec.R & "," & ClockColorSec.G & "," & ClockColorSec.B)
            sw.WriteLine("ClockDimSec=" & ClockDimSec)
            sw.WriteLine("ClockColorMin=" & ClockColorMin.R & "," & ClockColorMin.G & "," & ClockColorMin.B)
            sw.WriteLine("ClockDimMin=" & ClockDimMin)
            sw.WriteLine("ClockColorHour=" & ClockColorHour.R & "," & ClockColorHour.G & "," & ClockColorHour.B)
            sw.WriteLine("ClockDimHour=" & ClockDimHour)
            sw.WriteLine("DelayHour=" & DelayHour)
            sw.Flush()
            sw.Close()
        End Using
    End Sub
#End Region

End Class
