﻿Imports System.Drawing
Imports System.Drawing.Imaging
Imports System.IO
Imports System.Windows.Forms
Imports System.Runtime.InteropServices
Imports Microsoft.WindowsMobile.Status

Public Class Widget
    'Get the directory of this Widget
    Private DirWidget As String = Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().GetName().CodeBase)

    'Scale Device based on VGA Resolution
    Private DimScale As Single = 1

    'Dimension of Widget in VGA size(480x640)
    Private WidthWidget As Integer = 100
    Private HeightWidget As Integer = 50

    'Default Variables
    Private BatteryBmp As Bitmap
    Private bgBattery As Bitmap
    Private bgRed As Bitmap
    Private bgYellow As Bitmap
    Private bgGreen As Bitmap

    'Colors
    Private brsBlack As New SolidBrush(Color.Black)
    Private brsWhite As New SolidBrush(Color.White)

    'Font Battery
    Private fntBattery As Font
    Private StrForm As New StringFormat

    'Last refresh
    Private LastRefresh As New DateTime(1900, 1, 1)

    'Varibles to Battery informations
    <DllImport("coredll")> _
    Private Shared Function GetSystemPowerStatusEx(ByRef sps As SYSTEM_POWER_STATUS_EX) As Boolean
    End Function

    Private Structure SYSTEM_POWER_STATUS_EX
        Public ACLineStatus As Byte
        Public BatteryFlag As Byte
        Public BatteryLifePercent As Byte
        Public Reserved1 As Byte
        Public BatteryLifeTime As Integer
        Public BatteryFullLifeTime As Integer
        Public Reserved2 As Byte
        Public BackupBatteryFlag As Byte
        Public BackupBatteryLifePercent As Byte
        Public Reserved3 As Byte
        Public BackupBatteryLifeTime As Integer
        Public BackupBatteryFullLifeTime As Integer
    End Structure

    Private SPS As New SYSTEM_POWER_STATUS_EX

    'Variable on charging
    Private PercOnCharge As Integer = 0

#Region "Required Methods in this order"
    'Load variables for this Widget
    Public Sub LoadWidget(ByVal PathSettingsFile As String)
        Try
            'Set DimScale
            DimScale = Math.Max(480 / Math.Min(Screen.PrimaryScreen.Bounds.Width, Screen.PrimaryScreen.Bounds.Height), 640 / Math.Max(Screen.PrimaryScreen.Bounds.Width, Screen.PrimaryScreen.Bounds.Height))

            'Load bitmap
            bgBattery = New Bitmap(DirWidget & "\gfx\bg.png")
            bgRed = New Bitmap(DirWidget & "\gfx\red.png")
            bgYellow = New Bitmap(DirWidget & "\gfx\yellow.png")
            bgGreen = New Bitmap(DirWidget & "\gfx\green.png")

            'Set font
            fntBattery = New Font("Arial", 10 * DimScale, FontStyle.Bold)

            'Set String Format
            StrForm.Alignment = StringAlignment.Center

        Catch ex As Exception
            MsgBox(ex.Message, MsgBoxStyle.Critical, "Load Widget Error")
        End Try
    End Sub

    Public Sub DrawWidget(ByRef g As Graphics, ByVal Name As String, ByVal Position As Rectangle, ByVal IA As ImageAttributes, ByVal Inflate As Single)
        Try
            'If difference form now and last refresh is major of 1 second
            If DateDiff(DateInterval.Second, LastRefresh, Now) >= 1 Then
                'Update last refresh
                LastRefresh = Now

                'Check if Device is in charge
                Dim OnCharge As Boolean = (SystemState.PowerBatteryState = BatteryState.Charging)

                'If the Device is in charge refresh ever
                If OnCharge Then
                    'Change the dimension of font
                    If fntBattery.Size <> (6 * DimScale) Then fntBattery = New Font(fntBattery.Name, 6 * DimScale, fntBattery.Style)
                Else
                    'Change the dimension of font
                    If fntBattery.Size <> (10 * DimScale) Then fntBattery = New Font(fntBattery.Name, 10 * DimScale, fntBattery.Style)
                End If

                'Create new bitmap
                If BatteryBmp IsNot Nothing Then
                    BatteryBmp.Dispose()
                    BatteryBmp = Nothing
                End If
                BatteryBmp = New Bitmap(WidthWidget, HeightWidget)
                Using gx As Graphics = Graphics.FromImage(BatteryBmp)
                    'Draw Background Battery
                    gx.DrawImage(bgBattery, New Rectangle(0, 0, BatteryBmp.Width, BatteryBmp.Height), New Rectangle(0, 0, bgBattery.Width, bgBattery.Height), GraphicsUnit.Pixel)

                    'Create variables for text
                    Dim TextCharge As String
                    Dim ValCharge As Integer

                    'Get Status Battery
                    Try
                        GetSystemPowerStatusEx(SPS)
                    Catch ex As Exception
                    End Try

                    'Set Percent of BatteryLife
                    ValCharge = SPS.BatteryLifePercent

                    If OnCharge Then
                        PercOnCharge += (2 / DimScale)
                        If PercOnCharge > ValCharge Then PercOnCharge = 0
                        TextCharge = "In charging" & Environment.NewLine & "(" & ValCharge & "%)"
                    Else
                        PercOnCharge = ValCharge
                        TextCharge = ValCharge & "%"
                    End If

                    'Draw rectangle of current power status
                    Dim bmpCharge As Bitmap
                    Select Case ValCharge
                        Case Is < 20
                            bmpCharge = bgRed
                        Case Is < 60
                            bmpCharge = bgYellow
                        Case Else
                            bmpCharge = bgGreen
                    End Select

                    'Fill the battery
                    gx.DrawImage(bmpCharge, New Rectangle(3, 3, (PercOnCharge * 90) / 100, 44), New Rectangle(0, 0, bmpCharge.Width, bmpCharge.Height), GraphicsUnit.Pixel)

                    'Write Text of power status
                    gx.DrawString(TextCharge, fntBattery, brsBlack, 50 - (2 / DimScale), 6, StrForm)
                    gx.DrawString(TextCharge, fntBattery, brsBlack, 50 + (2 / DimScale), 6, StrForm)
                    gx.DrawString(TextCharge, fntBattery, brsBlack, 50, 6 - (2 / DimScale), StrForm)
                    gx.DrawString(TextCharge, fntBattery, brsBlack, 50, 6 + (2 / DimScale), StrForm)
                    gx.DrawString(TextCharge, fntBattery, brsWhite, 50, 6, StrForm)
                End Using
            End If

            'Draw widget
            IA.SetColorKey(BatteryBmp.GetPixel(0, 0), BatteryBmp.GetPixel(0, 0))
            g.DrawImage(BatteryBmp, Position, 0, 0, BatteryBmp.Width, BatteryBmp.Height, GraphicsUnit.Pixel, IA)

        Catch ex As Exception
            MsgBox(ex.Message, MsgBoxStyle.Critical, "Draw Widget Error")
        End Try
    End Sub

    Public Sub DisposeWidget()
        Try
            'Dispose objects
            SPS = Nothing

            If BatteryBmp IsNot Nothing Then
                BatteryBmp.Dispose()
                BatteryBmp = Nothing
            End If

            If bgBattery IsNot Nothing Then
                bgBattery.Dispose()
                bgBattery = Nothing
            End If

            If bgRed IsNot Nothing Then
                bgRed.Dispose()
                bgRed = Nothing
            End If

            If bgYellow IsNot Nothing Then
                bgYellow.Dispose()
                bgYellow = Nothing
            End If

            If bgGreen IsNot Nothing Then
                bgGreen.Dispose()
                bgGreen = Nothing
            End If

        Catch ex As Exception
            MsgBox(ex.Message, MsgBoxStyle.Critical, "Dispose Widget Error")
        End Try
    End Sub

    Public Sub ClickOnWidget(ByVal Position As Rectangle, ByVal PointMouse As Point)
        Try

            'Start the battery Control Panel
            Process.Start("ctlpnl", "cplmain.cpl,3")

        Catch ex As Exception
            MsgBox(ex.Message, MsgBoxStyle.Critical, "Click Widget Error")
        End Try
    End Sub

    'Return the dimension of widget
    Public Function GetDimension(ByVal WidthHeight As String) As Integer
        Try
            Select Case WidthHeight.ToLower
                Case "width"
                    Return WidthWidget
                Case "height"
                    Return HeightWidget

            End Select
        Catch ex As Exception
            MsgBox(ex.Message, MsgBoxStyle.Critical, "Get Dimension Widget Error")
        End Try
    End Function

    'Get the vertical scroll on widget
    Public Sub ScrollWidget(ByVal ScrollY As Integer)
        Try

        Catch ex As Exception
            MsgBox(ex.Message, MsgBoxStyle.Critical, "Error")
        End Try
    End Sub
#End Region

#Region "Other Functions"
    
#End Region

End Class
