﻿Imports System.Drawing
Imports System.Drawing.Imaging
Imports System.IO
Imports System.Windows.Forms

Public Class Widget
    'Get the directory of this Widget
    Private DirWidget As String = Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().GetName().CodeBase)
    Private DirGfx As String = DirWidget & "\gfx"

    'Scale Device based on VGA Resolution
    Private DimScale As Single = 1

    'Dimension of Widget in VGA size(480x640)
    Private WidthWidget As Integer = 200
    Private HeightWidget As Integer = 100

    'Path of Settings file loaded in LoadWidget
    Private SettingsFile As String

    'Default Variables 
    Private BGClock As Bitmap

    'Bitmap to draw every minute
    Private LastTime As String
    Private bmpClock As Bitmap

    'Fonts
    Private fntTime As Font
    Private fntDate As Font
    Private brsWhite As New SolidBrush(Color.White)
    Private brsBlack As New SolidBrush(Color.Black)
    Private strForm As New StringFormat

#Region "Required Methods in this order"
    'Load variables for this Widget
    Public Sub LoadWidget(ByVal PathSettingsFile As String)
        Try
            'Set the path of Settings File
            SettingsFile = PathSettingsFile

            'Load the Background Bitmap
            BGClock = New Bitmap(DirGfx & "\bg.png")

            'Create Bitmap of Clock
            bmpClock = New Bitmap(BGClock.Width, BGClock.Height)

            'Set DimScale
            DimScale = Math.Max(480 / Math.Min(Screen.PrimaryScreen.Bounds.Width, Screen.PrimaryScreen.Bounds.Height), 640 / Math.Max(Screen.PrimaryScreen.Bounds.Width, Screen.PrimaryScreen.Bounds.Height))

            'Set Fonts
            fntTime = New Font("Tahoma", 28 * DimScale, FontStyle.Regular)
            fntDate = New Font("Tahoma", 7 * DimScale, FontStyle.Regular)

        Catch ex As Exception
            MsgBox(ex.Message, MsgBoxStyle.Critical, "Error")
        End Try
    End Sub

    Public Sub DrawWidget(ByRef g As Graphics, ByVal Name As String, ByVal Position As Rectangle, ByVal IA As ImageAttributes, ByVal Inflate As Single)
        Try
            'If minute change
            If LastTime <> Format(Now, "HH.mm") Then
                'Update minute
                LastTime = Format(Now, "HH.mm")

                Using gx As Graphics = Graphics.FromImage(bmpClock)
                    'Draw the Clock Background
                    gx.DrawImage(BGClock, 0, 0)

                    'Calculate Time
                    Dim TimeNow As String = Format(Now, "HH:mm").Replace(".", ":")
                    Dim DateNow As String = Format(Now, "d-MMM-yyyy")

                    'If there's AM/PM i rewrite the time
                    If Now.ToLongTimeString.ToLower.Contains("am") Or Now.ToLongTimeString.ToLower.Contains("pm") Then
                        Select Case Now.Hour
                            Case Is > 12
                                TimeNow = (Now.Hour - 12) & ":" & Right("00" & Now.Minute, 2)
                            Case 0
                                TimeNow = "12:" & Right("00" & Now.Minute, 2)
                        End Select
                    End If

                    'Check the size of time string
                    Dim sizTime As Size = gx.MeasureString(TimeNow, fntTime).ToSize

                    'Set Centered String Format
                    strForm.Alignment = StringAlignment.Center

                    'Check where write Time
                    Dim TimeY As Integer = -10
                    gx.DrawString(TimeNow, fntTime, brsBlack, bmpClock.Width / 2, TimeY - 2, strForm)
                    gx.DrawString(TimeNow, fntTime, brsWhite, bmpClock.Width / 2, TimeY, strForm)

                    'If there's AM/PM i write IT
                    If Now.ToLongTimeString.ToLower.Contains("am") Or Now.ToLongTimeString.ToLower.Contains("pm") Then
                        'Set Left String Format
                        strForm.Alignment = StringAlignment.Near

                        'Write Date
                        Dim DatePos As New Point(10, TimeY + sizTime.Height - 6)
                        gx.DrawString(DateNow, fntDate, brsBlack, DatePos.X, DatePos.Y - 2, strForm)
                        gx.DrawString(DateNow, fntDate, brsWhite, DatePos.X, DatePos.Y, strForm)

                        'Set Right String Format
                        strForm.Alignment = StringAlignment.Far

                        'Reset X position
                        DatePos.X = 190

                        If Now.ToLongTimeString.ToLower.Contains("am") Then
                            gx.DrawString("AM", fntDate, brsBlack, DatePos.X, DatePos.Y - 2, strForm)
                            gx.DrawString("AM", fntDate, brsWhite, DatePos.X, DatePos.Y, strForm)

                        ElseIf Now.ToLongTimeString.ToLower.Contains("pm") Then
                            gx.DrawString("PM", fntDate, brsBlack, DatePos.X, DatePos.Y - 2, strForm)
                            gx.DrawString("PM", fntDate, brsWhite, DatePos.X, DatePos.Y, strForm)
                        End If
                    Else
                        'Write Date at center
                        Dim DatePos As New Point(bmpClock.Width / 2, TimeY + sizTime.Height - 6)
                        gx.DrawString(DateNow, fntDate, brsBlack, DatePos.X, DatePos.Y - 2, strForm)
                        gx.DrawString(DateNow, fntDate, brsWhite, DatePos.X, DatePos.Y, strForm)
                    End If
                End Using
            End If

            'Draw Bitmap Clock
            IA.SetColorKey(bmpClock.GetPixel(0, 0), bmpClock.GetPixel(0, 0))
            g.DrawImage(bmpClock, Position, 0, 0, bmpClock.Width, bmpClock.Height, GraphicsUnit.Pixel, IA)

        Catch ex As Exception
            MsgBox(ex.Message, MsgBoxStyle.Critical, "Error")
        End Try
    End Sub

    Public Sub DisposeWidget()
        Try
            'Dispose objects
            If BGClock IsNot Nothing Then
                BGClock.Dispose()
                BGClock = Nothing
            End If

            If bmpClock IsNot Nothing Then
                bmpClock.Dispose()
                bmpClock = Nothing
            End If

        Catch ex As Exception
            MsgBox(ex.Message, MsgBoxStyle.Critical, "Error")
        End Try
    End Sub

    Public Sub ClickOnWidget(ByVal Position As Rectangle, ByVal PointMouse As Point)
        Try
            'If you click on clock you open setting
            Process.Start("ctlpnl", "cplmain.cpl,16")
        Catch ex As Exception
            MsgBox(ex.Message, MsgBoxStyle.Critical, "Error")
        End Try
    End Sub

    'Return the dimension of widget
    Public Function GetDimension(ByVal WidthHeight As String) As Integer
        Try
            Select Case WidthHeight.ToLower
                Case "width"
                    Return WidthWidget
                Case "height"
                    Return HeightWidget

            End Select
        Catch ex As Exception
            MsgBox(ex.Message, MsgBoxStyle.Critical, "Error")
        End Try
    End Function

    'Get the vertical scroll on widget
    Public Sub ScrollWidget(ByVal ScrollY As Integer)
        Try

        Catch ex As Exception
            MsgBox(ex.Message, MsgBoxStyle.Critical, "Error")
        End Try
    End Sub
#End Region

#Region "Other Functions"

#End Region

End Class
