﻿Imports System.Drawing
Imports System.Drawing.Imaging
Imports System.IO
Imports System.Windows.Forms

Public Class Widget
    'Get the directory of this Widget
    Private DirWidget As String = Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().GetName().CodeBase)
    Private DirGfx As String = DirWidget & "\gfx"

    'Scale Device based on VGA Resolution
    Private DimScale As Single = 1

    'Dimension of Widget in VGA size(480x640)
    Private WidthWidget As Integer = 400
    Private HeightWidget As Integer = 240

    'Path of Settings file loaded in LoadWidget
    Private SettingsFile As String

    'Default Variables 
    Private BGClock As Bitmap
    Private Numbers(9) As Bitmap
    Private AMPM(1) As Bitmap

    'Bitmap to draw every minute
    Private LastTime As String
    Private bmpClock As Bitmap

    'Fonts
    Private fntDate As Font
    Private brsWhite As New SolidBrush(Color.White)
    Private brsBlack As New SolidBrush(Color.Black)
    Private strForm As New StringFormat

#Region "Required Methods in this order"
    'Load variables for this Widget
    Public Sub LoadWidget(ByVal PathSettingsFile As String)
        Try
            'Set the path of Settings File
            SettingsFile = PathSettingsFile

            'Load the Background Bitmap
            BGClock = New Bitmap(DirGfx & "\bg.png")

            'Create Bitmap Clock
            bmpClock = New Bitmap(BGClock.Width, BGClock.Height)

            'Load numbers
            For x As Integer = 0 To 9
                Numbers(x) = New Bitmap(DirGfx & "\" & x & ".png")
            Next

            'Load AM/PM
            AMPM(0) = New Bitmap(DirGfx & "\a.png")
            AMPM(1) = New Bitmap(DirGfx & "\p.png")

            'Set Centered String Format
            strForm.Alignment = StringAlignment.Center

            'Set DimScale
            DimScale = Math.Max(480 / Math.Min(Screen.PrimaryScreen.Bounds.Width, Screen.PrimaryScreen.Bounds.Height), 640 / Math.Max(Screen.PrimaryScreen.Bounds.Width, Screen.PrimaryScreen.Bounds.Height))

            'Set Fonts
            fntDate = New Font("Tahoma", 10 * DimScale, FontStyle.Regular)

        Catch ex As Exception
            MsgBox(ex.Message, MsgBoxStyle.Critical, "Error")
        End Try
    End Sub

    Public Sub DrawWidget(ByRef g As Graphics, ByVal Name As String, ByVal Position As Rectangle, ByVal IA As ImageAttributes, ByVal Inflate As Single)
        Try
            'If minute change
            If LastTime <> Format(Now, "HH.mm") Then
                'Update minute
                LastTime = Format(Now, "HH.mm")

                Using gx As Graphics = Graphics.FromImage(bmpClock)
                    'Draw the Clock Background
                    gx.DrawImage(BGClock, 0, 0)

                    'Calculate Time
                    Dim DateNow As String = Format(Now, "dddd, d MMMM")

                    'Draw bitmap of numbers of hour time
                    Dim Hr As Integer = Now.Hour
                    'If there is AM/PM in datetime i correct the Hour
                    If Now.ToLongTimeString.ToLower.Contains("am") Or Now.ToLongTimeString.ToLower.Contains("pm") Then
                        Select Case Hr
                            Case Is > 12
                                Hr -= 12
                            Case 0
                                Hr = 12
                        End Select
                    End If
                    'If hour is < 10 i draw only the hour in center
                    If Hr < 10 Then
                        gx.DrawImage(Numbers(Hr), 62, 32)
                    Else
                        'Else i draw the 2 numbers in center
                        Dim Hr1 As Integer = CInt(Hr.ToString.Substring(0, 1))
                        Dim Hr2 As Integer = CInt(Hr.ToString.Substring(1, 1))
                        gx.DrawImage(Numbers(Hr1), 22, 32)
                        gx.DrawImage(Numbers(Hr2), 102, 32)
                    End If

                    'If there's AM/PM i draw it
                    If Now.ToLongTimeString.ToLower.Contains("am") Then
                        gx.DrawImage(AMPM(0), 22, 140)
                    ElseIf Now.ToLongTimeString.ToLower.Contains("pm") Then
                        gx.DrawImage(AMPM(1), 22, 140)
                    End If

                    'Draw bitmap of numbers of minute time
                    Dim Mn As Integer = Now.Minute
                    Dim Mn1 As Integer = 0
                    Dim Mn2 As Integer = 0
                    If Mn >= 10 Then
                        Mn1 = CInt(Mn.ToString.Substring(0, 1))
                        Mn2 = CInt(Mn.ToString.Substring(1, 1))
                    Else
                        Mn2 = Mn
                    End If
                    gx.DrawImage(Numbers(Mn1), 222, 32)
                    gx.DrawImage(Numbers(Mn2), 302, 32)

                    'Check where write Date
                    gx.DrawString(DateNow, fntDate, brsBlack, bmpClock.Width / 2, 198, strForm)
                    gx.DrawString(DateNow, fntDate, brsWhite, bmpClock.Width / 2, 200, strForm)
                End Using
            End If

            'Draw Bitmap Clock
            IA.SetColorKey(bmpClock.GetPixel(0, 0), bmpClock.GetPixel(0, 0))
            g.DrawImage(bmpClock, Position, 0, 0, bmpClock.Width, bmpClock.Height, GraphicsUnit.Pixel, IA)

        Catch ex As Exception
            MsgBox(ex.Message, MsgBoxStyle.Critical, "Error")
        End Try
    End Sub

    Public Sub DisposeWidget()
        Try
            'Dispose objects
            If BGClock IsNot Nothing Then
                BGClock.Dispose()
                BGClock = Nothing
            End If

            For x As Integer = 0 To 9
                If Numbers(x) IsNot Nothing Then
                    Numbers(x).Dispose()
                    Numbers(x) = Nothing
                End If
            Next

            Numbers = Nothing

            For x As Integer = 0 To 1
                If AMPM(x) IsNot Nothing Then
                    AMPM(x).Dispose()
                    AMPM(x) = Nothing
                End If
            Next

            AMPM = Nothing

            If bmpClock IsNot Nothing Then
                bmpClock.Dispose()
                bmpClock = Nothing
            End If

        Catch ex As Exception
            MsgBox(ex.Message, MsgBoxStyle.Critical, "Error")
        End Try
    End Sub

    Public Sub ClickOnWidget(ByVal Position As Rectangle, ByVal PointMouse As Point)
        Try
            'If you click on clock you open setting
            Process.Start("ctlpnl", "cplmain.cpl,16")
        Catch ex As Exception
            MsgBox(ex.Message, MsgBoxStyle.Critical, "Error")
        End Try
    End Sub

    'Return the dimension of widget
    Public Function GetDimension(ByVal WidthHeight As String) As Integer
        Try
            Select Case WidthHeight.ToLower
                Case "width"
                    Return WidthWidget
                Case "height"
                    Return HeightWidget

            End Select
        Catch ex As Exception
            MsgBox(ex.Message, MsgBoxStyle.Critical, "Error")
        End Try
    End Function

    'Get the vertical scroll on widget
    Public Sub ScrollWidget(ByVal ScrollY As Integer)
        Try
            
        Catch ex As Exception
            MsgBox(ex.Message, MsgBoxStyle.Critical, "Error")
        End Try
    End Sub
#End Region

#Region "Other Functions"

#End Region

End Class
