﻿Imports System.Drawing
Imports System.Drawing.Imaging
Imports System.IO
Imports System.Windows.Forms
Imports System.Runtime.InteropServices
Imports Microsoft.WindowsMobile.Status

Public Class Widget
    'Get the directory of this Widget
    Private DirWidget As String = Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().GetName().CodeBase)

    'Scale Device based on VGA Resolution
    Private DimScale As Single = 1

    'Dimension of Widget in VGA size(480x640)
    Private WidthWidget As Integer = 100
    Private HeightWidget As Integer = 50

    'Default Variables
    Private LineBmp As Bitmap
    Private bgLine As Bitmap

    'Colors
    Private brsBlack As New SolidBrush(Color.Black)
    Private brsWhite As New SolidBrush(Color.White)

    'Font Battery
    Private fntLine As Font
    Private StrForm As New StringFormat

    'Last refresh
    Private LastRefresh As New DateTime(1900, 1, 1)

#Region "Required Methods in this order"
    'Load variables for this Widget
    Public Sub LoadWidget(ByVal PathSettingsFile As String)
        Try
            'Set DimScale
            DimScale = Math.Max(480 / Math.Min(Screen.PrimaryScreen.Bounds.Width, Screen.PrimaryScreen.Bounds.Height), 640 / Math.Max(Screen.PrimaryScreen.Bounds.Width, Screen.PrimaryScreen.Bounds.Height))

            'Load bitmap
            bgLine = New Bitmap(DirWidget & "\gfx\bg.png")

            'Set font
            fntLine = New Font("Arial", 5 * DimScale, FontStyle.Regular)

            'Set String Format
            StrForm.Alignment = StringAlignment.Center

        Catch ex As Exception
            MsgBox(ex.Message, MsgBoxStyle.Critical, "Load Widget Error")
        End Try
    End Sub

    Public Sub DrawWidget(ByRef g As Graphics, ByVal Name As String, ByVal Position As Rectangle, ByVal IA As ImageAttributes, ByVal Inflate As Single)
        Try
            'If difference from now and last refresh is major of ChangeTime
            If DateDiff(DateInterval.Second, LastRefresh, Now) >= 2 Then
                'Update last refresh
                LastRefresh = Now

                'Create new bitmap
                If LineBmp IsNot Nothing Then
                    LineBmp.Dispose()
                    LineBmp = Nothing
                End If
                LineBmp = New Bitmap(WidthWidget, HeightWidget)
                Using gx As Graphics = Graphics.FromImage(LineBmp)
                    'Clear background
                    gx.Clear(Color.Gray)

                    'Set StateLine
                    Dim StateLine As Byte = 0

                    'If no operator name dont check the signal
                    If SystemState.PhoneOperatorName <> String.Empty Then
                        Select Case SystemState.PhoneSignalStrength
                            Case Is > 90
                                StateLine = 5
                            Case Is > 70
                                StateLine = 4
                            Case Is > 50
                                StateLine = 3
                            Case Is > 30
                                StateLine = 2
                            Case Is > 0
                                StateLine = 1
                        End Select
                    End If
                    Dim LineDim As New Size(bgLine.Width, bgLine.Height / 6)

                    IA.SetColorKey(bgLine.GetPixel(0, 0), bgLine.GetPixel(0, 0))

                    If StateLine = 0 Then
                        gx.DrawImage(bgLine, New Rectangle(0, 0, LineDim.Width, LineDim.Height), 0, 0, LineDim.Width, LineDim.Height, GraphicsUnit.Pixel, IA)
                    Else
                        gx.DrawImage(bgLine, New Rectangle(0, 0, LineDim.Width, LineDim.Height), 0, LineDim.Height * StateLine, LineDim.Width, LineDim.Height, GraphicsUnit.Pixel, IA)
                    End If

                    'Scrivo l'operatore
                    Dim OPName As String = String.Empty
                    If SystemState.PhoneOperatorName <> String.Empty Then OPName = SystemState.PhoneOperatorName.ToUpper
                    If OPName = String.Empty Then OPName = "NO SERVICE"

                    'Write Text of operator
                    gx.DrawString(OPName, fntLine, brsBlack, 50 - (2 / DimScale), 34, StrForm)
                    gx.DrawString(OPName, fntLine, brsBlack, 50 + (2 / DimScale), 34, StrForm)
                    gx.DrawString(OPName, fntLine, brsBlack, 50, 34 - (2 / DimScale), StrForm)
                    gx.DrawString(OPName, fntLine, brsBlack, 50, 34 + (2 / DimScale), StrForm)
                    gx.DrawString(OPName, fntLine, brsWhite, 50, 34, StrForm)
                End Using
            End If

            'Draw widget
            IA.SetColorKey(LineBmp.GetPixel(0, 0), LineBmp.GetPixel(0, 0))
            g.DrawImage(LineBmp, Position, 0, 0, LineBmp.Width, LineBmp.Height, GraphicsUnit.Pixel, IA)

        Catch ex As Exception
            MsgBox(ex.Message, MsgBoxStyle.Critical, "Draw Widget Error")
        End Try
    End Sub

    Public Sub DisposeWidget()
        Try
            'Dispose objects
            If LineBmp IsNot Nothing Then
                LineBmp.Dispose()
                LineBmp = Nothing
            End If

            If bgLine IsNot Nothing Then
                bgLine.Dispose()
                bgLine = Nothing
            End If

        Catch ex As Exception
            MsgBox(ex.Message, MsgBoxStyle.Critical, "Dispose Widget Error")
        End Try
    End Sub

    Public Sub ClickOnWidget(ByVal Position As Rectangle, ByVal PointMouse As Point)
        Try

            'Start the battery Control Panel
            Process.Start("ctlpnl", "cplmain.cpl,20")

        Catch ex As Exception
            MsgBox(ex.Message, MsgBoxStyle.Critical, "Click Widget Error")
        End Try
    End Sub

    'Return the dimension of widget
    Public Function GetDimension(ByVal WidthHeight As String) As Integer
        Try
            Select Case WidthHeight.ToLower
                Case "width"
                    Return WidthWidget
                Case "height"
                    Return HeightWidget

            End Select
        Catch ex As Exception
            MsgBox(ex.Message, MsgBoxStyle.Critical, "Get Dimension Widget Error")
        End Try
    End Function

    'Get the vertical scroll on widget
    Public Sub ScrollWidget(ByVal ScrollY As Integer)
        Try

        Catch ex As Exception
            MsgBox(ex.Message, MsgBoxStyle.Critical, "Error")
        End Try
    End Sub
#End Region

#Region "Other Functions"

#End Region

End Class
