﻿Imports System.Drawing
Imports System.Drawing.Imaging
Imports System.IO
Imports Microsoft.WindowsMobile.Status
Imports Microsoft.WindowsMobile.PocketOutlook
Imports System.Windows.Forms

Public Class Widget
    'Get the directory of this Widget
    Private DirWidget As String = Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().GetName().CodeBase)
    Private DirGfx As String = DirWidget & "\gfx"

    'Scale Device based on VGA Resolution
    Private DimScale As Single = 1

    'Dimension of Widget in VGA size(480x640)
    Private WidthWidget As Integer = 400
    Private HeightWidget As Integer = 100

    'Path of Settings file loaded in LoadWidget
    Private SettingsFile As String

    'Default Variables 
    Private BGApp As Bitmap

    'Bitmap to draw
    Private LastTime As DateTime = DateTime.MinValue
    Private bmpApp As Bitmap

    'Fonts
    Private fntDay As Font
    Private fntDayN As Font
    Private fntHour As Font
    Private fntApp As Font
    Private brsWhite As New SolidBrush(Color.White)
    Private brsBlack As New SolidBrush(Color.Black)
    Private strForm As New StringFormat

    'Appointment
    Private App As Appointment = Nothing

#Region "Required Methods in this order"
    'Load variables for this Widget
    Public Sub LoadWidget(ByVal PathSettingsFile As String)
        Try
            'Set the path of Settings File
            SettingsFile = PathSettingsFile

            'Load the Background Bitmap
            BGApp = New Bitmap(DirGfx & "\bg.png")

            'Create Bitmap
            bmpApp = New Bitmap(BGApp.Width, BGApp.Height)

            'Set next Appointment
            GetNextApp()

            'Set DimScale
            DimScale = Math.Max(480 / Math.Min(Screen.PrimaryScreen.Bounds.Width, Screen.PrimaryScreen.Bounds.Height), 640 / Math.Max(Screen.PrimaryScreen.Bounds.Width, Screen.PrimaryScreen.Bounds.Height))

            'Set Fonts
            fntDay = New Font("Tahoma", 5 * DimScale, FontStyle.Bold)
            fntDayN = New Font("Tahoma", 18 * DimScale, FontStyle.Bold)
            fntHour = New Font("Tahoma", 8 * DimScale, FontStyle.Bold)
            fntApp = New Font("Tahoma", 8 * DimScale, FontStyle.Regular)

            'Set Centered String Format
            strForm.Alignment = StringAlignment.Center

        Catch ex As Exception
            MsgBox(ex.Message, MsgBoxStyle.Critical, "Error")
        End Try
    End Sub

    Public Sub DrawWidget(ByRef g As Graphics, ByVal Name As String, ByVal Position As Rectangle, ByVal IA As ImageAttributes, ByVal Inflate As Single)
        Try
            'Every 1 seconds
            If DateDiff(DateInterval.Second, LastTime, Now) >= 1 Then
                LastTime = Now

                'Get next appointment()
                Try
                    GetNextApp()
                Catch ex As Exception
                    LastTime = String.Empty
                End Try

                Using gx As Graphics = Graphics.FromImage(bmpApp)
                    'Draw Background
                    gx.DrawImage(BGApp, 0, 0)

                    If App Is Nothing Then
                        Dim ToDay As String = Format(Now, "ddd").ToUpper
                        Dim ToDayN As String = Now.Day

                        'Draw Day name
                        gx.DrawString(ToDay, fntDay, brsWhite, 52, 11, strForm)

                        'Draw Day number
                        gx.DrawString(ToDayN, fntDayN, brsBlack, 52, 18, strForm)

                        'Draw Message
                        gx.DrawString("No next appointments", fntHour, brsWhite, 120, 40)
                    Else
                        'Calculate Date Time
                        Dim TimeStart As String = Format(App.Start, "HH:mm").Replace(".", ":")
                        Dim TimeEnd As String = Format(App.End, "HH:mm").Replace(".", ":")
                        Dim AppDay As String = Format(App.Start, "ddd").ToUpper
                        Dim AppDayN As String = App.Start.Day

                        'If there's AM/PM i rewrite the time
                        If Now.ToLongTimeString.ToLower.Contains("am") Or Now.ToLongTimeString.ToLower.Contains("pm") Then
                            Select Case App.Start.Hour
                                Case Is > 12
                                    TimeStart = (App.Start.Hour - 12) & ":" & Right("00" & App.Start.Minute, 2) & " pm"
                                Case 0
                                    TimeStart = "12:" & Right("00" & App.Start.Minute, 2) & " am"
                                Case Else
                                    TimeStart &= IIf(App.Start.ToLongTimeString.ToLower.Contains("am"), " am", " pm")
                            End Select

                            Select Case App.End.Hour
                                Case Is > 12
                                    TimeEnd = (App.End.Hour - 12) & ":" & Right("00" & App.End.Minute, 2) & " pm"
                                Case 0
                                    TimeEnd = "12:" & Right("00" & App.End.Minute, 2) & " am"
                                Case Else
                                    TimeEnd &= IIf(App.End.ToLongTimeString.ToLower.Contains("am"), " am", " pm")
                            End Select
                        End If

                        'Draw Day name
                        gx.DrawString(AppDay, fntDay, brsWhite, 52, 11, strForm)

                        'Draw Day number
                        gx.DrawString(AppDayN, fntDayN, brsBlack, 52, 18, strForm)

                        'Draw Appointment
                        gx.DrawString(TimeStart & " - " & TimeEnd, fntHour, brsWhite, 120, 25)
                        Dim TxtApp As String = App.Subject
                        If App.Location <> String.Empty Then TxtApp &= " (" & App.Location & ")"
                        If TxtApp.Length > 30 Then TxtApp = TxtApp.Substring(0, 27) & "..."
                        gx.DrawString(TxtApp, fntApp, brsWhite, 120, 50)
                    End If
                End Using
            End If

            'Draw Bitmap
            IA.SetColorKey(bmpApp.GetPixel(0, 0), bmpApp.GetPixel(0, 0))
            g.DrawImage(bmpApp, Position, 0, 0, bmpApp.Width, bmpApp.Height, GraphicsUnit.Pixel, IA)

        Catch ex As Exception
            MsgBox(ex.Message, MsgBoxStyle.Critical, "Error")
        End Try
    End Sub

    Public Sub DisposeWidget()
        Try
            'Dispose objects
            If BGApp IsNot Nothing Then
                BGApp.Dispose()
                BGApp = Nothing
            End If

            If bmpApp IsNot Nothing Then
                bmpApp.Dispose()
                bmpApp = Nothing
            End If

        Catch ex As Exception
            MsgBox(ex.Message, MsgBoxStyle.Critical, "Error")
        End Try
    End Sub

    'PointMouse contains the relative coords to position rectangle
    Public Sub ClickOnWidget(ByVal Position As Rectangle, ByVal PointMouse As Point)
        Try
            'If you click on it you open Appointment
            If App IsNot Nothing Then
                'Show appointment
                App.ShowDialog()
                LastTime = DateTime.MinValue
            Else
                'Show calendar
                Process.Start("calendar.exe", "")
            End If

        Catch ex As Exception
            MsgBox(ex.Message, MsgBoxStyle.Critical, "Error")
        End Try
    End Sub

    'Return the dimension of widget
    Public Function GetDimension(ByVal WidthHeight As String) As Integer
        Try
            Select Case WidthHeight.ToLower
                Case "width"
                    Return WidthWidget
                Case "height"
                    Return HeightWidget

            End Select
        Catch ex As Exception
            MsgBox(ex.Message, MsgBoxStyle.Critical, "Error")
        End Try
    End Function

    'Get the vertical scroll on widget
    Public Sub ScrollWidget(ByVal ScrollY As Integer)
        Try

        Catch ex As Exception
            MsgBox(ex.Message, MsgBoxStyle.Critical, "Error")
        End Try
    End Sub
#End Region

#Region "Other Functions"
    Private Sub GetNextApp()
        Try
            'With SystemState Get Next Appointment if is near
            App = SystemState.CalendarNextAppointment
        Catch ex As Exception
            App = Nothing
        End Try
    End Sub

#End Region

End Class
