﻿Imports System.IO
Imports System.Windows.Forms

Public Class FolderBrowseDialog
    Inherits Form

#Region "Variables"
    'Some constants here
    Const ROOT As String = "\"
    Const GOBACK As String = ".."
    Const WindowsPrograms As String = "Windows Programs ->"

    'Variables
    Public CurrentPath As String = ROOT
    Private Updating As Boolean = False

    'Program scale
    Private DimScale As Single

    'Collection of rows
    Private CollRows As Collections.Generic.SortedList(Of String, ListViewItem)

    'enum for the file system ListViewItems ImageIndex
    'This is just for readibility
    Private Enum FileSystemIcon As Integer
        File = 0
        Directory
        StorageCard
        Back
    End Enum

    'enum for the filesystem type
    Private Enum FileSystemKind
        File = 1
        Directory
        StorageCard
    End Enum
#End Region

#Region "Events"
    ' Only the basic constructor is implemented
    ' Feel free to implement your own overrides
    Public Sub New()
        InitializeComponent()
    End Sub

    Protected Overloads Overrides Sub OnLoad(ByVal e As EventArgs)
        MyBase.OnLoad(e)

        'Set scale
        DimScale = Math.Max(480 / Math.Min(Screen.PrimaryScreen.Bounds.Width, Screen.PrimaryScreen.Bounds.Height), 640 / Math.Max(Screen.PrimaryScreen.Bounds.Width, Screen.PrimaryScreen.Bounds.Height))

        'Reset dimension icons
        FileSystemIcons.ImageSize = New Drawing.Size(FileSystemIcons.ImageSize.Width / DimScale, FileSystemIcons.ImageSize.Height / DimScale)

        UpdateControls()
    End Sub

    'When the user activates an item
    'we can browse the directory or open the file....
    Private Sub fileListView_ItemActivate(ByVal sender As Object, ByVal e As EventArgs) Handles fileListView.ItemActivate
        Dim list As ListView = TryCast(sender, ListView)

        'We'll continue only if there is a selected item
        If list IsNot Nothing AndAlso list.SelectedIndices.Count > 0 Then
            Dim currentItem As ListViewItem = list.Items(list.SelectedIndices(0))
            Dim selItem As FileSystemKind = Nothing

            Try
                selItem = DirectCast(currentItem.Tag, FileSystemKind)
            Catch ex As Exception
            End Try

            'If it's not a file we should change the current path
            If selItem = Nothing Or selItem <> FileSystemKind.File Then
                If currentItem.Text = GOBACK Then
                    CurrentPath = CurrentPath.Substring(0, CurrentPath.Length - Path.GetFileName(CurrentPath).Length - 1)
                Else
                    CurrentPath = Path.Combine(CurrentPath, currentItem.Text)
                End If
                UpdateControls()
            End If
        End If
    End Sub

    ' We can change the current path using the PathSelectorComboBox
    Private Sub PathSelectorComboBox_SelectedIndexChanged(ByVal sender As Object, ByVal e As EventArgs) Handles PathSelectorComboBox.SelectedIndexChanged
        'Only do this if the view is not being updated
        If Not Updating Then
            '1) Build the new path
            CurrentPath = String.Empty

            'Se è stato scelto di andare ai Windows Programs ci vado direttamente
            If PathSelectorComboBox.Text = WindowsPrograms Then
                CurrentPath = Environment.GetFolderPath(Environment.SpecialFolder.StartMenu)
            Else
                For i As Integer = 0 To TryCast(sender, ComboBox).SelectedIndex
                    CurrentPath = Path.Combine(CurrentPath, PathSelectorComboBox.Items(i).ToString())
                Next
            End If

            '2) Update the dialog!
            UpdateControls()
        End If
    End Sub

    'Go to the parent folder by softkey1
    Private Sub mniSelect_Click(ByVal sender As Object, ByVal e As EventArgs) Handles mniSelect.Click
        Close()
    End Sub

    'CANCEL
    'Cancel by softkey2
    Private Sub mniCancel_Click(ByVal sender As Object, ByVal e As EventArgs) Handles mniCancel.Click
        CurrentPath = String.Empty
        Close()
    End Sub

#End Region

#Region "Functions"
    ' Refill the ComboBox with the path
    ' step by step
    Private Sub UpdateSelector()
        PathSelectorComboBox.Items.Clear()
        Dim pathParts As String() = CurrentPath.Split(Path.DirectorySeparatorChar)

        For Each part As String In pathParts
            If part = String.Empty Then
                If Not PathSelectorComboBox.Items.Contains("\") Then
                    PathSelectorComboBox.Items.Add("\")
                End If
            Else
                PathSelectorComboBox.Items.Add(part)
            End If
        Next
        PathSelectorComboBox.SelectedIndex = PathSelectorComboBox.Items.Count - 1

        'Aggiungo la voce per andare direttamente a \Windows\Start Menu
        PathSelectorComboBox.Items.Add(WindowsPrograms)
    End Sub

    ' Refill the file list
    Private Sub UpdateList()
        fileListView.SuspendLayout()

        ' 1) Clear the list and normalize the current path
        fileListView.Items.Clear()
        If CurrentPath = String.Empty Then CurrentPath = ROOT
        If CurrentPath.Length > 33 Then
            FullDir.Text = "..." & CurrentPath.Substring(CurrentPath.Length - 30, 30)
        Else
            FullDir.Text = CurrentPath
        End If

        'Ricreo la lista di righe
        If CollRows IsNot Nothing Then CollRows.Clear()
        CollRows = New Collections.Generic.SortedList(Of String, ListViewItem)

        ' 2) Add 'Back' if needed
        If CurrentPath <> ROOT Then
            Dim item As New ListViewItem(GOBACK)
            item.ImageIndex = CInt(FileSystemIcon.Back)
            fileListView.Items.Add(item)
        End If

        ' 3) Add directories and/or Storage Cards
        For Each entry As String In Directory.GetDirectories(CurrentPath)
            Dim item As New ListViewItem(Path.GetFileName(entry))
            Dim info As New FileInfo(entry)
            Dim kind As FileSystemKind = GetKindFor(info)
            Select Case kind
                Case FileSystemKind.StorageCard
                    item.ImageIndex = CInt(FileSystemIcon.StorageCard)
                    Exit Select
                Case FileSystemKind.Directory
                    item.ImageIndex = CInt(FileSystemIcon.Directory)
                    Exit Select
            End Select
            item.Tag = kind

            'Aggiungo alla collection la directory
            CollRows.Add("0-" & item.Text, item)
        Next

        ' Aggiungo le righe alla lista
        For Each lst As ListViewItem In CollRows.Values
            fileListView.Items.Add(lst)
        Next

        fileListView.ResumeLayout()
    End Sub

    'Helper method which find out the kind of file system entry
    'based on the attributes
    Private Function GetKindFor(ByVal info As FileSystemInfo) As FileSystemKind
        If (info.Attributes And (FileAttributes.Directory Or FileAttributes.Temporary)) = (FileAttributes.Directory Or FileAttributes.Temporary) Then
            Return FileSystemKind.StorageCard
        ElseIf (info.Attributes And FileAttributes.Directory) = FileAttributes.Directory Then
            Return FileSystemKind.Directory
        Else
            Return FileSystemKind.File
        End If
    End Function

    'Update the dialog!
    Private Sub UpdateControls()
        If Updating Then Exit Sub

        Updating = True

        Try
            Cursor.Current = Cursors.WaitCursor
            UpdateList()
            UpdateSelector()
        Finally
            Updating = False
            Cursor.Current = Cursors.[Default]
        End Try
    End Sub
#End Region

End Class