﻿Imports System.Drawing
Imports System.Drawing.Imaging
Imports System.IO
Imports System.Windows.Forms

Public Class Widget
    'Get the directory of this Widget
    Private DirWidget As String = Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().GetName().CodeBase)

    'Scale Device based on VGA Resolution
    Private DimScale As Single = 1

    'Dimension of Widget in VGA size(480x640)
    Private WidthWidget As Integer = 200
    Private HeightWidget As Integer = 200

    'Path of Settings file loaded in LoadWidget
    Private SettingsFile As String

    'Default Variables 
    Private WidgetBmp As Bitmap
    Private BorderBmp As Bitmap
    Private SettingsBmp As Bitmap
    Private btnSettingsSize As Size
    Private NextBmp As Bitmap
    Private btnNextSize As Size
    Private BorderImm As String = "Border1.png"
    Private PathImms As String = Environment.GetFolderPath(Environment.SpecialFolder.Personal)
    Private RandomImm As Boolean = True
    Private ChangeTime As Integer = 60

    'Colors
    Private brsBlack As New SolidBrush(Color.Black)

    'Last change image time
    Private LastChangeTime As New DateTime(1900, 1, 1)

    'Current image visible
    Private CurrentImage As String = String.Empty

#Region "Required Methods in this order"
    'Load variables for this Widget
    Public Sub LoadWidget(ByVal PathSettingsFile As String)
        Try
            'Set the path of Settings File
            SettingsFile = PathSettingsFile

            'Load Settings
            LoadSettings()

            'Load the Border Bitmap
            BorderBmp = New Bitmap(DirWidget & "\Borders\" & BorderImm)

            'Load Buttons Bitmap
            SettingsBmp = New Bitmap(DirWidget & "\gfx\Settings.png")
            NextBmp = New Bitmap(DirWidget & "\gfx\Next.png")

            'Set DimScale
            DimScale = Math.Max(480 / Math.Min(Screen.PrimaryScreen.Bounds.Width, Screen.PrimaryScreen.Bounds.Height), 640 / Math.Max(Screen.PrimaryScreen.Bounds.Width, Screen.PrimaryScreen.Bounds.Height))

            'Set buttons size
            btnSettingsSize = New Size(50 / DimScale, 50 / DimScale)
            btnNextSize = New Size(50 / DimScale, 50 / DimScale)

        Catch ex As Exception
            MsgBox(ex.Message, MsgBoxStyle.Critical, "Error")
        End Try
    End Sub

    Public Sub DrawWidget(ByRef g As Graphics, ByVal Name As String, ByVal Position As Rectangle, ByVal IA As ImageAttributes, ByVal Inflate As Single)
        Try
            'If difference form now and last change time is major of change time
            If DateDiff(DateInterval.Second, LastChangeTime, Now) >= ChangeTime Then
                'Update last change time
                LastChangeTime = Now

                'Fill collections of images
                Dim collImms As New Collections.Generic.SortedList(Of String, String)
                For Each fil As String In Directory.GetFiles(PathImms)
                    Select Case Path.GetExtension(fil).ToLower
                        Case ".jpg", ".jpeg", ".png", ".gif", ".bmp"
                            collImms.Add(fil, Path.GetFileName(fil))
                    End Select
                Next

                'Create bitmap for PhotoFrame
                If WidgetBmp IsNot Nothing Then
                    WidgetBmp.Dispose()
                    WidgetBmp = Nothing
                End If
                WidgetBmp = New Bitmap(Position.Width, Position.Height)
                Using gx As Graphics = Graphics.FromImage(WidgetBmp)
                    Dim SizOffImm As New Size(Int(WidgetBmp.Width / BorderBmp.Width) + 1, Int(WidgetBmp.Height / BorderBmp.Height) + 1)

                    'Clear background
                    gx.Clear(Color.Fuchsia)
                    gx.FillRectangle(brsBlack, SizOffImm.Width, SizOffImm.Height, WidgetBmp.Width - (SizOffImm.Width * 2), WidgetBmp.Height - (SizOffImm.Height * 2))

                    'If I find images
                    If collImms.Count > 0 Then
                        'Get random image in collection
                        If RandomImm Then
                            CurrentImage = collImms.Keys(New Random().Next(0, collImms.Count - 1))
                        Else
                            'If CurrentImage is empty set first image
                            If CurrentImage = String.Empty Then
                                CurrentImage = collImms.Keys(0)
                            Else
                                'Get next image in collection
                                For x As Integer = 0 To collImms.Count - 1
                                    If collImms.Keys(x) = CurrentImage Then
                                        If x < (collImms.Count - 1) Then
                                            CurrentImage = collImms.Keys(x + 1)
                                        Else
                                            'Set first image in collection
                                            CurrentImage = collImms.Keys(0)
                                        End If
                                        'Exit from for because i'm in last position
                                        Exit For
                                    End If
                                Next
                            End If
                        End If

                        'Draw current image
                        Try
                            Using bmp As New Bitmap(CurrentImage)
                                Dim DestPhoto As Rectangle = ZoomImage(New Size(bmp.Width, bmp.Height), New Size(WidgetBmp.Width - (SizOffImm.Width * 2), WidgetBmp.Height - (SizOffImm.Height * 2)))
                                'Move rectangle to correct position
                                DestPhoto.Offset(SizOffImm.Width, SizOffImm.Height)
                                'Draw bmp
                                gx.DrawImage(bmp, DestPhoto, New Rectangle(0, 0, bmp.Width, bmp.Height), GraphicsUnit.Pixel)
                            End Using
                        Catch ex As Exception
                        End Try
                    End If

                    'Draw border
                    IA.SetColorKey(BorderBmp.GetPixel(0, 0), BorderBmp.GetPixel(0, 0))
                    gx.DrawImage(BorderBmp, New Rectangle(0, 0, Position.Width, Position.Height), 0, 0, BorderBmp.Width, BorderBmp.Height, GraphicsUnit.Pixel, IA)

                    'Set the buttons to draw
                    Dim rectSettings As New Rectangle((Position.Width - (btnSettingsSize.Width + btnNextSize.Width)) / 2, Position.Height - btnSettingsSize.Height, btnSettingsSize.Width, btnSettingsSize.Height)
                    Dim rectNext As New Rectangle(rectSettings.Right, Position.Height - btnNextSize.Height, btnNextSize.Width, btnNextSize.Height)

                    'Draw Buttons
                    IA.SetColorKey(SettingsBmp.GetPixel(0, 0), SettingsBmp.GetPixel(0, 0))
                    gx.DrawImage(SettingsBmp, rectSettings, 0, 0, SettingsBmp.Width, SettingsBmp.Height, GraphicsUnit.Pixel, IA)
                    IA.SetColorKey(NextBmp.GetPixel(0, 0), NextBmp.GetPixel(0, 0))
                    gx.DrawImage(NextBmp, rectNext, 0, 0, NextBmp.Width, NextBmp.Height, GraphicsUnit.Pixel, IA)
                End Using
            End If

            'Draw widget
            IA.SetColorKey(WidgetBmp.GetPixel(0, 0), WidgetBmp.GetPixel(0, 0))
            g.DrawImage(WidgetBmp, Position, 0, 0, WidgetBmp.Width, WidgetBmp.Height, GraphicsUnit.Pixel, IA)

        Catch ex As Exception
            MsgBox(ex.Message, MsgBoxStyle.Critical, "Error")
        End Try
    End Sub

    Public Sub DisposeWidget()
        Try
            'Dispose objects
            If WidgetBmp IsNot Nothing Then
                WidgetBmp.Dispose()
                WidgetBmp = Nothing
            End If

            If BorderBmp IsNot Nothing Then
                BorderBmp.Dispose()
                BorderBmp = Nothing
            End If

            If SettingsBmp IsNot Nothing Then
                SettingsBmp.Dispose()
                SettingsBmp = Nothing
            End If

            If NextBmp IsNot Nothing Then
                NextBmp.Dispose()
                NextBmp = Nothing
            End If

        Catch ex As Exception
            MsgBox(ex.Message, MsgBoxStyle.Critical, "Error")
        End Try
    End Sub

    Public Sub ClickOnWidget(ByVal Position As Rectangle, ByVal PointMouse As Point)
        Try
            'Set rectangle for buttons relative to 0,0 position
            Dim rectSettings As New Rectangle((Position.Width - (btnSettingsSize.Width + btnNextSize.Width)) / 2, Position.Height - btnSettingsSize.Height, btnSettingsSize.Width, btnSettingsSize.Height)
            Dim rectNext As New Rectangle(rectSettings.Right, Position.Height - btnNextSize.Height, btnNextSize.Width, btnNextSize.Height)

            'Debug
            'MsgBox(Position.X & "," & Position.Y & "," & Position.Width & "," & Position.Height & "; " & PointMouse.X & "," & PointMouse.Y)
            'MsgBox(rectSettings.X & "," & rectSettings.Y & "," & rectSettings.Width & "," & rectSettings.Height & "; " & PointMouse.X & "," & PointMouse.Y)
            'Exit Sub

            'If click on next reset the time to load next picture
            If rectNext.Contains(PointMouse) Then
                LastChangeTime = New DateTime(1900, 1, 1)

            ElseIf rectSettings.Contains(PointMouse) Then 'If click in settings rectangle open the form
                Using frm As New Settings
                    With frm
                        'Load files to border images
                        For Each fil As String In Directory.GetFiles(DirWidget & "\Borders", "*.png")
                            .BorderImm.Items.Add(Path.GetFileName(fil))
                        Next

                        'Set fields form
                        .BorderImm.Text = BorderImm
                        .PathImms.Text = PathImms
                        .RandomImm.Checked = RandomImm
                        .ChangeTime.Value = ChangeTime

                        'Show form
                        .ShowDialog()

                        'If user click on Ok button, save data
                        If .SaveData Then
                            BorderImm = .BorderImm.Text
                            PathImms = .PathImms.Text
                            RandomImm = .RandomImm.Checked
                            ChangeTime = .ChangeTime.Value

                            'Save settings
                            SaveSettings()

                            'Reload bitmap
                            If BorderBmp IsNot Nothing Then
                                BorderBmp.Dispose()
                                BorderBmp = Nothing
                            End If
                            BorderBmp = New Bitmap(DirWidget & "\Borders\" & BorderImm)

                            'Reset Last Change Time
                            LastChangeTime = New DateTime(1900, 1, 1)
                        End If
                    End With
                End Using

            Else
                'If click on widget open the image viewed
                If CurrentImage <> String.Empty Then Process.Start(CurrentImage, "")
            End If

        Catch ex As Exception
            MsgBox(ex.Message, MsgBoxStyle.Critical, "Error")
        End Try
    End Sub

    'Return the dimension of widget
    Public Function GetDimension(ByVal WidthHeight As String) As Integer
        Try
            Select Case WidthHeight.ToLower
                Case "width"
                    Return WidthWidget
                Case "height"
                    Return HeightWidget

            End Select
        Catch ex As Exception
            MsgBox(ex.Message, MsgBoxStyle.Critical, "Error")
        End Try
    End Function

    'Get the vertical scroll on widget
    Public Sub ScrollWidget(ByVal ScrollY As Integer)
        Try

        Catch ex As Exception
            MsgBox(ex.Message, MsgBoxStyle.Critical, "Error")
        End Try
    End Sub
#End Region

#Region "Other Functions"
    'Load the Settings
    Private Sub LoadSettings()
        'If settings file not exist
        If Not File.Exists(SettingsFile) Then
            'I create it saving the default variables
            SaveSettings()
        Else
            'Read the settings file
            Using sr As New StreamReader(SettingsFile)
                'Read first line
                Dim row As String = sr.ReadLine

                'Process all lines
                Do While row IsNot Nothing
                    'If this row isnt empty or it isnt a comment i process it
                    If row.Trim <> String.Empty AndAlso Not row.StartsWith("//") AndAlso Not row.StartsWith("'") Then
                        'Split Setting and Value
                        Dim setting As String = row.ToLower.Split("=")(0)
                        Dim val As String = row.ToLower.Split("=")(1)

                        'Check which value is to set
                        Select Case setting
                            Case "borderimm"
                                BorderImm = val

                            Case "pathimms"
                                PathImms = val

                            Case "randomimm"
                                RandomImm = val

                            Case "changetime"
                                ChangeTime = val

                        End Select
                    End If

                    'Read next line
                    row = sr.ReadLine
                Loop
            End Using
        End If

    End Sub

    'Save the settings
    Private Sub SaveSettings()
        'Write the settings file
        Using sw As New StreamWriter(SettingsFile, False)
            sw.WriteLine("BorderImm=" & BorderImm)
            sw.WriteLine("PathImms=" & PathImms)
            sw.WriteLine("RandomImm=" & RandomImm)
            sw.WriteLine("ChangeTime=" & ChangeTime)
            sw.Flush()
            sw.Close()
        End Using
    End Sub

    'Zoom image maintaining the proportions
    Public Function ZoomImage(ByVal ImageSize As Size, ByVal SpaceSize As Size) As Rectangle
        Dim NewHeight As Integer
        Dim NewWidth As Integer
        If ImageSize.Width > ImageSize.Height Then
            If SpaceSize.Width > SpaceSize.Height Then
                NewHeight = (ImageSize.Height * SpaceSize.Width) \ ImageSize.Width
                If NewHeight > SpaceSize.Height Then NewHeight = SpaceSize.Height
                NewWidth = (ImageSize.Width * NewHeight) \ ImageSize.Height
            Else
                NewWidth = (ImageSize.Width * SpaceSize.Height) \ ImageSize.Height
                If NewWidth > SpaceSize.Width Then NewWidth = SpaceSize.Width
                NewHeight = (ImageSize.Height * NewWidth) \ ImageSize.Width
            End If
        Else
            If SpaceSize.Width > SpaceSize.Height Then  'Altezza Schermo + piccola
                NewWidth = (ImageSize.Width * SpaceSize.Height) \ ImageSize.Height
                If NewWidth > SpaceSize.Width Then NewWidth = SpaceSize.Width
                NewHeight = (ImageSize.Height * NewWidth) \ ImageSize.Width
            Else
                NewHeight = (ImageSize.Height * SpaceSize.Width) \ ImageSize.Width
                If NewHeight > SpaceSize.Height Then NewHeight = SpaceSize.Height
                NewWidth = (ImageSize.Width * NewHeight) \ ImageSize.Height
            End If
        End If

        Return New Rectangle((SpaceSize.Width - NewWidth) \ 2, (SpaceSize.Height - NewHeight) \ 2, NewWidth, NewHeight)
    End Function
#End Region

End Class
